#ifndef EEPROM_CONFIG_H
#define EEPROM_CONFIG_H

#define EEPROM_BASE_ADDR ((uint8_t*)0x08080000)

// boolean flags. Each bit represents one flag.
#define EEPROM_CONFIG_FLAGS     (*(uint32_t*)(EEPROM_BASE_ADDR + 0x0000))

// flag definitions
#define EEPROM_CONFIG_FLAG_USE_DCDC_CONTROL  (1 << 0)

// corridor definition
#define EEPROM_CONFIG_CORRIDOR_MIN_TEMPERATURE     (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0004))
#define EEPROM_CONFIG_CORRIDOR_MAX_TEMPERATURE     (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0008))

// The fan is started when temperature rises above this temperature.
#define EEPROM_CONFIG_FAN_START_TEMPERATURE        (*(fxp_t*)(EEPROM_BASE_ADDR + 0x000C))

// The fan is stopped when the temperature falls below this point.
#define EEPROM_CONFIG_FAN_OFF_TEMPERATURE          (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0010))

// Emergency temperature level. If this is reached, the duty cycle is directly
// set to the maximum to skip a long ramp-up phase.
#define EEPROM_CONFIG_EMERGENCY_TEMPERATURE        (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0014))

// The fan PWM duty cycle cannot fall below this limit. It is held until
// temperature falls below the off-temperature.
#define EEPROM_CONFIG_DUTY_CYCLE_MIN               (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0018))

// The fan PWM duty cycle cannot go above this limit. Should be set to 100.
#define EEPROM_CONFIG_DUTY_CYCLE_MAX               (*(fxp_t*)(EEPROM_BASE_ADDR + 0x001C))

// The fan is always started with this duty cycle. Do not set too low.
#define EEPROM_CONFIG_DUTY_CYCLE_START             (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0020))

// gain unit: percent duty cycle change per °C deviation per update cycle
#define EEPROM_CONFIG_GAIN_T_HIGH_P                (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0024))
#define EEPROM_CONFIG_GAIN_T_HIGH_I                (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0028))
#define EEPROM_CONFIG_GAIN_T_LOW_P                 (*(fxp_t*)(EEPROM_BASE_ADDR + 0x002C))
#define EEPROM_CONFIG_GAIN_T_LOW_I                 (*(fxp_t*)(EEPROM_BASE_ADDR + 0x0030))

#endif // EEPROM_CONFIG_H
